@@ -18,20 +18,21 @@
 """Utilities and helper functions."""
 import base64
 import binascii
+import contextlib
 import eventlet
 import functools
 import inspect
+import math
 import mimetypes
 
+from neutron_lib import constants as n_const
 from oslo_concurrency import lockutils
 from oslo_concurrency import processutils
 from oslo_context import context as common_context
 from oslo_log import log as logging
-from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import strutils
 import pecan
-import six
 
 from zun.api import utils as api_utils
 from zun.common import clients
@@ -57,7 +58,7 @@ VALID_STATES = {
     'delete_force': [consts.CREATED, consts.CREATING, consts.ERROR,
                      consts.RUNNING, consts.STOPPED, consts.UNKNOWN,
                      consts.DELETED, consts.DEAD, consts.RESTARTING,
-                     consts.REBUILDING],
+                     consts.REBUILDING, consts.DELETING],
     'delete_after_stop': [consts.RUNNING, consts.CREATED, consts.ERROR,
                           consts.STOPPED, consts.DELETED, consts.DEAD],
     'start': [consts.CREATED, consts.STOPPED, consts.ERROR],
@@ -89,6 +90,7 @@ VALID_STATES = {
 }
 
 VALID_CONTAINER_FILED = {
+    'name': 'name',
     'image': 'image',
     'command': 'command',
     'args': 'args',
@@ -98,6 +100,8 @@ VALID_CONTAINER_FILED = {
     'env': 'environment',
     'workDir': 'workdir',
     'imagePullPolicy': 'image_pull_policy',
+    'tty': 'tty',
+    'stdin': 'interactive',
 }
 
 VALID_CAPSULE_FIELD = {
@@ -134,7 +138,7 @@ def safe_rstrip(value, chars=None):
     :return: Stripped value.
 
     """
-    if not isinstance(value, six.string_types):
+    if not isinstance(value, str):
         LOG.warning(
             "Failed to remove trailing character. Returning original object. "
             "Supplied object is not a string: %s.", value)
@@ -222,9 +226,9 @@ def translate_exception(function):
             return function(self, context, *args, **kwargs)
         except Exception as e:
             if not isinstance(e, exception.ZunException):
-                LOG.exception("Unexpected error: %s", six.text_type(e))
+                LOG.exception("Unexpected error: %s", str(e))
                 e = exception.ZunException("Unexpected error: %s"
-                                           % six.text_type(e))
+                                           % str(e))
                 raise e
             raise
 
@@ -247,10 +251,7 @@ def check_container_id(function):
 
 def get_image_pull_policy(image_pull_policy, image_tag):
     if not image_pull_policy:
-        if image_tag == 'latest' or not image_tag:
-            image_pull_policy = 'always'
-        else:
-            image_pull_policy = 'ifnotpresent'
+        image_pull_policy = 'always'
     return image_pull_policy
 
 
@@ -336,7 +337,7 @@ def custom_execute(*cmd, **kwargs):
     except processutils.ProcessExecutionError as e:
         sanitized_cmd = strutils.mask_password(' '.join(cmd))
         raise exception.CommandError(cmd=sanitized_cmd,
-                                     error=six.text_type(e))
+                                     error=str(e))
 
 
 def get_root_helper():
@@ -364,36 +365,6 @@ def execute(*cmd, **kwargs):
         return custom_execute(*cmd, **kwargs)
 
 
-def check_capsule_template(tpl):
-    # TODO(kevinz): add volume spec check
-    tpl_json = tpl
-    if isinstance(tpl, six.string_types):
-        try:
-            tpl_json = jsonutils.loads(tpl)
-        except Exception as e:
-            raise exception.FailedParseStringToJson(e)
-
-    kind_field = tpl_json.get('kind')
-    if kind_field not in ['capsule', 'Capsule']:
-        raise exception.InvalidCapsuleTemplate("kind fields need to be "
-                                               "set as capsule or Capsule")
-
-    spec_field = tpl_json.get('spec')
-    if spec_field is None:
-        raise exception.InvalidCapsuleTemplate("No Spec found")
-    # Align the Capsule restartPolicy with container restart_policy
-    # Also change the template filed name from Kubernetes type to OpenStack
-    # type.
-    if 'restartPolicy' in spec_field.keys():
-        spec_field['restartPolicy'] = \
-            VALID_CAPSULE_RESTART_POLICY[spec_field['restartPolicy']]
-        spec_field[VALID_CAPSULE_FIELD['restartPolicy']] = \
-            spec_field.pop('restartPolicy')
-    if spec_field.get('containers') is None:
-        raise exception.InvalidCapsuleTemplate("No valid containers field")
-    return spec_field, tpl_json
-
-
 def capsule_get_container_spec(spec_field):
     containers_spec = spec_field.get('containers')
     containers_num = len(containers_spec)
@@ -408,8 +379,9 @@ def capsule_get_container_spec(spec_field):
                                                    "image is needed")
         # Remap the Capsule's container fields to native Zun container fields.
         for key in list(container_spec.keys()):
-            container_spec[VALID_CONTAINER_FILED[key]] = \
-                container_spec.pop(key)
+            if key in VALID_CONTAINER_FILED:
+                container_spec[VALID_CONTAINER_FILED[key]] = \
+                    container_spec.pop(key)
 
     init_containers_spec = spec_field.get('initContainers')
     if init_containers_spec is not None:
@@ -421,8 +393,9 @@ def capsule_get_container_spec(spec_field):
             # Remap the Capsule's container fields to native Zun
             # container fields.
             for key in list(container_spec.keys()):
-                container_spec[VALID_CONTAINER_FILED[key]] = \
-                    container_spec.pop(key)
+                if key in VALID_CONTAINER_FILED:
+                    container_spec[VALID_CONTAINER_FILED[key]] = \
+                        container_spec.pop(key)
     else:
         init_containers_spec = []
 
@@ -473,24 +446,6 @@ def is_all_projects(search_opts):
     return all_projects
 
 
-def get_container(container_ident):
-    container = api_utils.get_resource('Container', container_ident)
-    if not container:
-        pecan.abort(404, ('Not found; the container you requested '
-                          'does not exist.'))
-
-    return container
-
-
-def get_image(image_id):
-    image = api_utils.get_resource('Image', image_id)
-    if not image:
-        pecan.abort(404, ('Not found; the image you requested '
-                          'does not exist.'))
-
-    return image
-
-
 def get_registry(registry_id):
     registry = api_utils.get_resource('Registry', registry_id)
     if not registry:
@@ -632,6 +587,25 @@ class EventReporter(object):
         return False
 
 
+class FinishAction(object):
+    """Context manager to finish container actions."""
+
+    def __init__(self, context, action_name, *container_uuids):
+        self.context = context
+        self.action_name = action_name
+        self.container_uuids = container_uuids
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        for uuid in self.container_uuids:
+            objects.ContainerAction.action_finish(
+                self.context, uuid, self.action_name, exc_val=exc_val,
+                exc_tb=exc_tb, want_result=False)
+        return False
+
+
 def get_wrapped_function(function):
     """Get the method at the bottom of a stack of decorators."""
     if not hasattr(function, '__closure__') or not function.__closure__:
@@ -655,7 +629,7 @@ def get_wrapped_function(function):
     return _get_wrapped_function(function)
 
 
-def wrap_container_event(prefix):
+def wrap_container_event(prefix, finish_action=None):
     """Warps a method to log the event taken on the container, and result.
 
     This decorator wraps a method to log the start and result of an event, as
@@ -669,10 +643,18 @@ def wrap_container_event(prefix):
             keyed_args = inspect.getcallargs(wrapped_func, self, context,
                                              *args, **kwargs)
             container_uuid = keyed_args['container'].uuid
-
             event_name = '{0}_{1}'.format(prefix, function.__name__)
-            with EventReporter(context, event_name, container_uuid):
-                return function(self, context, *args, **kwargs)
+
+            if finish_action is not None:
+                with FinishAction(
+                    context, finish_action, container_uuid
+                ), EventReporter(
+                    context, event_name, container_uuid
+                ):
+                    return function(self, context, *args, **kwargs)
+            else:
+                with EventReporter(context, event_name, container_uuid):
+                    return function(self, context, *args, **kwargs)
         return decorated_function
     return helper
 
@@ -687,10 +669,10 @@ def wrap_exception():
             except exception.DockerError as e:
                 with excutils.save_and_reraise_exception(reraise=False):
                     LOG.error("Error occurred while calling Docker API: %s",
-                              six.text_type(e))
+                              str(e))
             except Exception as e:
                 with excutils.save_and_reraise_exception(reraise=False):
-                    LOG.exception("Unexpected exception: %s", six.text_type(e))
+                    LOG.exception("Unexpected exception: %s", str(e))
         return decorated_function
     return helper
 
@@ -707,7 +689,7 @@ def is_less_than(x, y):
 
 
 def encode_file_data(data):
-    if six.PY3 and isinstance(data, str):
+    if isinstance(data, str):
         data = data.encode('utf-8')
     return base64.b64encode(data).decode('utf-8')
 
@@ -718,3 +700,70 @@ def decode_file_data(data):
         return base64.b64decode(data)
     except (TypeError, binascii.Error):
         raise exception.Base64Exception()
+
+
+def strtime(at):
+    return at.strftime("%Y-%m-%dT%H:%M:%S.%f")
+
+
+@contextlib.contextmanager
+def nested_contexts(*contexts):
+    with contextlib.ExitStack() as stack:
+        yield [stack.enter_context(c) for c in contexts]
+
+
+def convert_mb_to_ceil_gb(mb_value):
+    gb_int = 0
+    if mb_value:
+        gb_float = mb_value / 1024.0
+        # ensure we reserve/allocate enough space by rounding up to nearest GB
+        gb_int = int(math.ceil(gb_float))
+    return gb_int
+
+
+if hasattr(inspect, 'getfullargspec'):
+    getargspec = inspect.getfullargspec
+else:
+    getargspec = inspect.getargspec
+
+
+def expects_func_args(*args):
+    def _decorator_checker(dec):
+        @functools.wraps(dec)
+        def _decorator(f):
+            base_f = get_wrapped_function(f)
+            argspec = getargspec(base_f)
+            if argspec[1] or argspec[2] or set(args) <= set(argspec[0]):
+                # NOTE (ndipanov): We can't really tell if correct stuff will
+                # be passed if it's a function with *args or **kwargs so
+                # we still carry on and hope for the best
+                return dec(f)
+            else:
+                raise TypeError("Decorated function %(f_name)s does not "
+                                "have the arguments expected by the "
+                                "decorator %(d_name)s" %
+                                {'f_name': base_f.__name__,
+                                 'd_name': dec.__name__})
+        return _decorator
+    return _decorator_checker
+
+
+def is_port_active(neutron_port):
+    """Checks if port is active.
+
+    :param neutron_port: dict containing port information as returned by
+                         neutron client's 'show_port'
+    """
+    return (neutron_port['status'] == n_const.PORT_STATUS_ACTIVE)
+
+
+def reraise(tp, value, tb=None):
+    try:
+        if value is None:
+            value = tp()
+        if value.__traceback__ is not tb:
+            raise value.with_traceback(tb)
+        raise value
+    finally:
+        value = None
+        tb = None
