@@ -14,23 +14,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import socket
-
 from oslo_config import cfg
 
 
-service_opts = [
-    cfg.HostAddressOpt('host',
-                       default=socket.gethostname(),
-                       sample_default='localhost',
-                       help='Name of this node. This can be an '
-                            'opaque identifier. It is not necessarily '
-                            'a hostname, FQDN, or IP address. '
-                            'However, the node name must be valid '
-                            'within an AMQP key, and if using ZeroMQ, '
-                            'a valid hostname, FQDN, or IP address.'),
-]
-
 periodic_opts = [
     cfg.IntOpt('periodic_interval_max',
                default=60,
@@ -40,9 +26,25 @@ periodic_opts = [
                default=180,
                help='Max interval size between periodic tasks execution in '
                     'seconds.'),
+    cfg.IntOpt('sync_container_state_interval',
+               default=60,
+               help="""
+Interval to sync container states between the database and the docker.
+
+The interval that Zun checks the actual container state and
+the state that Zun has recorded in its database. If they are inconsistent,
+Zun will update the database according to the actual container state.
+
+Possible values:
+* 0: Will run at the default periodic interval.
+* Any value < 0: Disables the option.
+* Any positive integer in seconds.
+
+"""),
 ]
 
-ALL_OPTS = (service_opts + periodic_opts)
+
+ALL_OPTS = (periodic_opts)
 
 
 def register_opts(conf):
